//=============================================================================
//  Lazarus 関係サンプル
//  Lazarus で作成したアプリの二重起動防止
//  全てメインフォーム側で処理する例
//
//-----------------------------------------------------------------------------
//
//  【使用条件等】
//
//  当サイトのアプリケーション類，コンポーネント，コード類は，転載，改変，配布
//  (オリジナル，改変版)などは全て自由です．商用利用も自由です．連絡，使用の記
//  載義務もありません．ただしオリジナルそのものを商用とすることは禁止します．
//  なお，本サイトのプログラムやコードを使用したことによって生じた，いかなる障
//  害，損失に関しても作者は一切関与しないものとします．ご了承下さい．
//
//-----------------------------------------------------------------------------
//
//  【履歴】
//
//  2020年07月01日 ～
//
//-----------------------------------------------------------------------------
//
//  【動作確認環境】
//
//  Windows 7 U64(SP1) + lazarus-2.0.8
//
//  Presented by Mr.XRAY
//  http://mrxray.on.coocan.jp/
//=============================================================================
unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, StdCtrls, Windows;

type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private

  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

const
  UniqueName:string = 'HogeHogeApp';

//-----------------------------------------------------------------------------
//  EnumWindows のコールバック関数
//  起動済みのアプリを最前面に表示するために使用
//
//  lazarus-2.0.8 では ShowWindow(hWindow, SW_RESTORE) は機能しない
//  代わりに WM_SYSCOMMAND で SC_RESTORE をポストしている
//-----------------------------------------------------------------------------
function EnumWndProc(hWindow: HWND; lData: LPARAM): BOOL;stdcall;
begin
  Result := True;
  if GetProp(hWindow, PChar(lData)) = 1111 then begin
    if IsIconic(hWindow) then begin
      PostMessage(hWindow, WM_SYSCOMMAND, $FFF0 and SC_RESTORE, 0);
    end;
    SetForegroundWindow(hWindow);
    Result := False;
  end;
end;

//=============================================================================
//  フォーム生成時の処理
//  二重起動防止のチェック
//  既に起動していたらメインフォームであるこのフォームを閉じる
//=============================================================================
procedure TForm1.FormCreate(Sender: TObject);
begin
  CreateMutex(nil, true, PChar(UniqueName));
  if GetLastError = ERROR_ALREADY_EXISTS then begin
    ShowMessage('このアプリは二重起動できません');
    EnumWindows(@EnumWndProc, LPARAM(PChar(UniqueName)));
    PostMessage(Handle, WM_CLOSE, 0, 0);
  end else begin
    SetProp(Application.Handle, PChar(UniqueName), 1111);
  end;
end;

//=============================================================================
//  メインフォーム破棄時の処理
//=============================================================================
procedure TForm1.FormDestroy(Sender: TObject);
begin
  RemoveProp(Application.Handle, PChar(UniqueName));
end;

//=============================================================================
//=============================================================================
procedure TForm1.Button1Click(Sender: TObject);
begin
//
end;

end.
